@@ -111,8 +111,7 @@ fn enumerate(
                 let budget = (budget.0 + p, budget.1 + p);
                 let depth = depth + 1;
                 let idx = (0, &expr);
-                let f = expr.clone();
-                enumerate_many(dsl, &ctx, env, f, idx, arg_tps, budget, p, depth, cb)
+                enumerate_many(dsl, &ctx, env, &expr, idx, arg_tps, budget, p, depth, cb)
             })
     }
 }
@@ -122,7 +121,7 @@ fn enumerate_many(
     dsl: &Language,
     ctx: &Context,
     env: &Rc<LinkedList<Type>>,
-    f: Expression,
+    f: &Expression,
     idx: (usize, &Expression),
     mut arg_tps: VecDeque<Type>,
     budget: (f64, f64),
@@ -143,7 +142,7 @@ fn enumerate_many(
             let arg_tps = arg_tps.clone();
             let budget = (budget.0 + ll, budget.1 + ll);
             let offset = offset + ll;
-            enumerate_many(dsl, &ctx, env, f, idx, arg_tps, budget, offset, depth, cb)
+            enumerate_many(dsl, &ctx, env, &f, idx, arg_tps, budget, offset, depth, cb)
         };
         enumerate(dsl, ctx, &arg_tp, env, (0f64, budget.1), depth, cb_arg)
     } else if budget.0 < 0f64 {
